==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/client/inc/obj_tree.js <==
/**
 * TREE object is visual object for ordering array data into tree structure.
 *
 * Input array:
 * x[<p item>][link] = 'index.html'
 *            [childs][<ch item>][link]
 *                               [childs]...
 *
 * All items have its own ID identificator
 * ID = obj_tree._pathName + <name> + / + <name>...
 *
 * EVN: _onactivate
 *      _onclick
 *
 * doresit autoukladani do datasetu
 **/
_me = obj_tree.prototype;
function obj_tree(){};

_me.__constructor = function(){
	this.__value = [];		// array of open nodes
	this._saveState = true;	// save __value into dataset saver
	this.__idtable;			// table of arguments mapped on each node
	this.__activeNode;
	this._tree;
	this._treeCookie;

	var me = this;

	this._main.onclick = function(e){
		var e = e || window.event;
		/* stop event propagation
		e.cancelBubble = true;
		if (e.stopPropagation) e.stopPropagation();
		*/

		// ELEMENT which caused event
		var elm = e.target || e.srcElement;
		var eli,id;

		if (elm.tagName == 'SPAN' && elm.parentNode.tagName == 'A')
			elm = elm.parentNode;

		switch(elm.tagName){
			case 'A'   :
				eli = searchParent(elm,'LI');
				id = eli.id.substr(me._pathName.length+1);

				if (e.type != 'contextmenu')
					me._setActive(id);

				var idt = me.__idtable[id]?clone(me.__idtable[id],true):{};

				if (me._onclick) me._onclick(e,elm,id,idt || {});
				me.__exeEvent('onclick',e,{"arg":idt,"id":id,"elm":elm,"owner":me});
				break;

			case 'SPAN':
				eli = searchParent(elm,'LI');
				id = eli.id.substr(me._pathName.length+1);
				me._open(id);
				break;
		}
		return false;
	};

	this._main.ondblclick = function(e){
		var e = e || window.event;
		var elm = e.target || e.srcElement;
		
		if (elm.tagName == 'A'){
			eli = searchParent(elm,'LI');
			id = eli.id.substr(me._pathName.length+1);
			me._open(id);
		}
	};

    this._main.oncontextmenu = this._main.onclick;

	/*
	this._main.oncontextmenu = function(e){
		var e = e || window.event;
	}
	*/
};


/**
 * @brief: redraw tree object
	[Events]
	.. [nodes]
	.. [arg]
	.... [aid] = admin@merakdemo.com
	.... [fid] = Events
	.... [ftype] = E
	.... [disabled] = false
	.. [text] = Calendar
	.. [title2] =
	.. [ico] = ico_e
	.. [itmclass] =
 *
 **/
_me._fill = function (aData){

	/* clear container object */
	this._main.innerHTML = '';

	/* retrieve data from dataset */
	if (!aData) aData = dataSet.get(this._listener_data,this._listenerPath_data);
	if (!aData) return;
	this.__row(aData);
};


_me._invalidateActive = function() {
	delete this.__activeNode;
};

_me._getActive = function() {
	if (typeof this.__activeNode == 'undefined')
		return Path.split('');
	else
		return Path.split(this.__activeNode);
}


/**
 * @brief: set active node and open thread
 **/
_me._setActive = function(id){

	try{
		var elm = document.getElementById(this._getElmId(id)).getElementsByTagName('a')[0];
		addcss(elm,'active');
		elm = null;
	}
	catch(e){
		return false;
	}

	if (this.__activeNode && this.__activeNode != id) {
		try{
			var elm2 = document.getElementById(this._getElmId(this.__activeNode)).getElementsByTagName('a')[0];
			removecss(elm2,'active');
			elm2 = null;
		}
		catch(e){}
	}

	// Open parent node
	var iSPos;
	if ((iSPos = id.lastIndexOf('/')) > -1)
		this._open(id.substr(0,iSPos),'minus');

	if (this._onactivate)
		this._onactivate(id);

	this.__activeNode = id;
	return true;
};


/**
 * @brief: return DOM Element ID
 **/
_me._getElmId = function (id){
	return this._pathName +'/'+ id;
};


/**
 * @brief: recursiv method for creating ul-li tree structure
 * @scope: PRIVATE
 * @date : 12.5.2006 11:50:37
 **/
_me.__row = function(aData,child){

	var path='',elm1,elm2,elm3,elm4,elm5,elm6,liclass,me = this;

	/* create main UL element */
	elm1 = mkElement("ul");
	if (!child){
		this.__idtable = {};
		elm1.className = 'fTree';
		liclass = 'root';
	}
	else{
		path = child;
		liclass = 'none';
	}

	for(var i in aData){
		elm2 = mkElement("li",{"className":liclass,"id":this._pathName + '/' + path + i});
		elm3 = mkElement("div");

		if (aData[i]['arg'])
			this.__idtable[path + i] = aData[i]['arg'];

		/* [+] icon */
		elm5 = mkElement("span",{"className":'plussign'});
		elm5.innerHTML = '&nbsp;&nbsp;&nbsp;';
		elm3.appendChild(elm5);

		/* folder icon */
		elm6 = mkElement("span",{"className": (aData[i]['ico'] ? 'ico ' + aData[i]['ico'] : 'ico') });
		elm6.innerHTML = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
		elm3.appendChild(elm6);

		/* link */
		if (aData[i]["arg"]["disabled"])
			elm4 = mkElement("span",{"className":'inactive'});
		else
			elm4 = mkElement("a",{"href":'',"className":'treeitem'});
		
		if (aData[i]["itmclass"])
			addcss(elm4,aData[i]["itmclass"]);

		elm4.innerHTML = (aData[i]['title']?getLang(aData[i]['title'],null,true):(aData[i]['text']?aData[i]['text']:i.toString().entityify())) + (aData[i]['title2']?'<span>'+aData[i]['title2']+'</span>':'');  // Quote STRING !!!

		elm3.appendChild(elm4);

		elm2.appendChild(elm3);

		if (aData[i]['nodes'] && !Is.Empty(aData[i]['nodes'])){
			/* set "minus" className for node with childs */
			addcss(elm2,'plus');
			removecss(elm2,'none');

			/* parse and append child nodes */
			elm2.appendChild(this.__row(aData[i]['nodes'],path + i + '/'));
		}
		// n++;
		elm1.appendChild(elm2);
	}

	/* set "end" className for last node with childs */
	elm2.className += ' end';

	/* destruct html elements */
	elm2 = null; elm3 = null; elm4 = null; elm5 = null; elm6 = null;

	/* append root UL into _main */
	if (!child){
		this._main.appendChild(elm1);

		/* open previously opened nodes */
		if (count(this.__value))
			this._value(this.__value);
		else
		if (this._listener)
			this.__update(this._listener);

		// active
		if (this.__activeNode)
			this._setActive(this.__activeNode);

		return;
	}

	/* return UL element */
	return elm1;
};


/**
 * @brief: apply given CSS to node and all parent nodes,
 *         if node is not set than function works over all nodes in __value
 * @scope: PUBLIC
 * @date : 14.5.2006 15:58:10
 **/
_me._openup = function(css,id){
	var a = [],elm,pos,styl;

	if (!id)
		for(var i in this.__value)
		    a.push(this.__value[i]);
	else
		a.push(id);

	css = css == 'minus'?css:'plus';

	for (var i = 0; i<a.length;i++){
		/* remove wrong paths from array */
		if (!(elm = document.getElementById(this._getElmId(a[i])))){

			if ((pos = a[i].lastIndexOf('/'))>-1 && inArray(a,a[i].substr(0,pos))<0)
				a.push(a[i].substr(0,pos));

			a.splice(i,1);
			i--;
			continue;
		}

		if ((pos = a[i].lastIndexOf('/'))>-1 && inArray(a,a[i].substr(0,pos))<0)
			a.push(a[i].substr(0,pos));

		/* open node and parent nodes */
		if (elm.tagName != 'LI') continue;

		if (elm.className.indexOf(css) == -1){

			if (elm.className.indexOf('minus')>-1){
				removecss(elm,'minus');
				addcss(elm,css);
			}
			else
			if (elm.className.indexOf('plus')>-1){
				removecss(elm,'plus');
				addcss(elm,css);
			}
		}
	}
};

/**
 * @brief: open given node
 * @scope: PUBLIC
 * @param: ID    - Node id
 *         style - private recursive
 * @date : 14.5.2006 17:33:06
 **/
_me._open = function(id,style){
	if(!id) return;
	var css,elm,elmid = this._getElmId(id),a = id.split("/");

	/* check whether element exists */
	if (!(elm = document.getElementById(elmid))) return;

	/* close Node */
	if (elm.className.indexOf('plus')>-1 || style == 'minus'){

		/* add node into __value array */
		this.__value[getFreeKey(this.__value)] = id;
		this.__clear();
		/* open parent Nodes */
		this._openup('minus',id);
	}
	/* open node */
	else
	if (elm.className.indexOf('minus')>-1 || style == 'plus'){
		removecss(elm,'minus');
		addcss(elm,'plus');

		/* close node and all child nodes */
		var e;
		for(var i in this.__value){
//if (this.__value[i]==id || this.__value[i].indexOf(id+"/")==0)
			if (this.__value[i]==id){
				try{
					e = document.getElementById(this._pathName+"/"+this.__value[i]);
					if (e.className.indexOf('minus')>-1){
						removecss(e,'minus');
						addcss(e,'plus');
					}
				}
				catch(e){}
				delete this.__value[i];
				break; //pridano
			}
		}

		/* if node is not root node than add parent node path into array */
/*
		if (a.length>1) {
			var t = a.slice(0,-1).join("/");
			if (inArray(this.__value,t)==-1)
				this.__value[getFreeKey(this.__value)] = t;
		}
*/
	}
	else{
		elm = null;
		if (a.length>1)
			this._open(a.slice(0,-1).join("/"),'minus');
		return;
	}

	/* destruct elm var */
	elm = null;
	/* save opened threads into dataset */
	if (this._saveState && this._listener)
		this._saveme();

	if (this._treeCookie)
	{
		this.__clear();
		dataSet.add(this._treeCookie,'',clone(this._value(),true));
	}
};

/**
 * @brief: listener for tree structure
 **/
_me._listen_data = function(sDataSet,aDataPath){
	this._listener_data = sDataSet;
	if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
	dataSet.obey(this,'_listener_data',sDataSet);
};

_me._listen_cookie = function(sCookieName)
{
	this._treeCookie = sCookieName;

	var aCookieValue = dataSet.get(sCookieName);
	if (typeof aCookieValue == 'object')
		this._value(aCookieValue);
};

/**
 * @brief: update method
 **/
_me.__update = function(sDataSet,aDataPath){

	/* no update section */
	if (this._norefresh){
		this._updateBuffer = true;
		return false;
	}
	else
		this._updateBuffer = false;

	/* update tree structure */
	if (this._listener_data == sDataSet){

		if (aDataPath && aDataPath[2] == 'RECENT'){

			var elm;
			if((elm = document.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1])) && (elm = elm.getElementsByTagName('A')[0]) && (elm = elm.getElementsByTagName('SPAN')[0])){
				var iRec = dataSet.get(sDataSet,aDataPath);
				if (parseInt(iRec)>0){
					addcss(elm.parentNode,'recent');
					elm.innerHTML = '&nbsp;('+iRec+')';
				}
				else{
					dataSet.remove(sDataSet,aDataPath,true);
                    removecss(elm.parentNode,'recent');
					elm.innerHTML = '';
				}
				
				elm = null;
				return;
			}
		}
		else
			this._fill();
	}

	/* update opened nodes */
	if (this._listener == sDataSet)
		this._value(dataSet.get(this._listener,this._listenerPath));
};


/**
 * @brief: return all open nodes (only last child in thread) as array of string paths
 * @scope: PUBLIC
 * @date : 14.5.2006 17:33:56
 **/
_me._value = function (v){
	if (v && typeof(v) == 'object') {
		/* close all nodes */
		if (this.__value.length) this._openup('plus');
		this.__value = this.__clear(v);
		/* open all nodes */
		this._openup('minus');
	}
	else
		this.__value = this.__clear(this.__value);

	return this.__value;
};

/**
 * @brief: removes spare paths from array
 * @scope: PRIVATE
 * @date : 14.5.2006 15:32:02
 **/
_me.__clear = function(a){
	if (!a)
		a = this.__value;

	a.sort().reverse();
	
	var pre;

	for(var i=0;i <= a.length;i++)
		if (typeof a[i] != 'undefined')
			if (typeof pre == 'undefined')
				pre=a[i];
			else
			if (pre==a[i])
				delete a[i];
			else
				pre = a[i];
	return a;
};

/**
 * @brief: optional onclick method
 * @param: e    - Event Handler (onclick event object)
 *         elm  - srcElement (link)
 *         id   - objects internal id (without pathName)
 *         arg  - item's arg section
 */
//_me._onclick = function(e,elm,id,arg){};

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/client/inc/obj_tree.js <==
/**
 * TREE object is visual object for ordering array data into tree structure.
 *
 * Input array:
 * x[<p item>][link] = 'index.html'
 *            [childs][<ch item>][link]
 *                               [childs]...
 *
 * All items have its own ID identificator
 * ID = obj_tree._pathName + <name> + / + <name>...
 *
 * EVN: _onactivate
 *      _onclick
 *
 * doresit autoukladani do datasetu
 **/
_me = obj_tree.prototype;
function obj_tree(){};

_me.__constructor = function(){

	this.__value = [];		// array of open nodes
	this._saveState = true;	// save __value into dataset saver
	this.__idtable;			// table of arguments mapped on each node
	this.__activeNode;
	this._tree;
	this._treeCookie;

	var me = this;

	this._main.onclick = function(e){
		var e = e || window.event;
		/* stop event propagation
		e.cancelBubble = true;
		if (e.stopPropagation) e.stopPropagation();
		*/

		// ELEMENT which caused event
		var elm = e.target || e.srcElement;
		var eli,id;

		if (elm.tagName == 'SPAN' && elm.parentNode.tagName == 'A')
			elm = elm.parentNode;

		switch(elm.tagName){
			case 'A'   :
				eli = searchParent(elm,'LI');
				id = eli.id.substr(me._pathName.length+1);

			//	alert(e.type);



                    
				if (e.type != 'contextmenu')
					me._setActive(id);

				var idt = me.__idtable[id]?clone(me.__idtable[id],true):{};

				if (me._onclick) me._onclick(e,elm,id,idt || {});
				me.__exeEvent('onclick',e,{"arg":idt,"id":id,"elm":elm,"owner":me});
				break;

			case 'SPAN':
				eli = searchParent(elm,'LI');
				id = eli.id.substr(me._pathName.length+1);
				me._open(id);
				break;
		}
		return false;
	};

	this._main.ondblclick = function(e){
		var e = e || window.event;
		var elm = e.target || e.srcElement;
		
		if (elm.tagName == 'A'){
			eli = searchParent(elm,'LI');
			id = eli.id.substr(me._pathName.length+1);
			me._open(id);
		}
	};

    this._main.oncontextmenu = this._main.onclick;
};


/**
 * @brief: redraw tree object
	[Events]
	.. [nodes]
	.. [arg]
	.... [aid] = admin@merakdemo.com
	.... [fid] = Events
	.... [ftype] = E
	.... [disabled] = false
	.. [text] = Calendar
	.. [title2] =
	.. [ico] = ico_e
	.. [itmclass] =
 *
 **/
_me._fill = function (aData){
	/* retrieve data from dataset */
	if (!aData) aData = dataSet.get(this._listener_data,this._listenerPath_data);
	if (!aData){
        // clear container object
        this._main.innerHTML = '';
		return;
	}
	this.__row(aData);
};


_me._invalidateActive = function() {
	delete this.__activeNode;
};

_me._getActive = function() {
	if (typeof this.__activeNode == 'undefined')
		return Path.split('');
	else
		return Path.split(this.__activeNode);
}


/**
 * @brief: set active node and open thread
 **/
_me._setActive = function(id){

	try{
		var elm = document.getElementById(this._getElmId(id)).getElementsByTagName('a')[0];
		addcss(elm,'active');
		elm = null;
	}
	catch(e){
		return false;
	}

	if (this.__activeNode && this.__activeNode != id) {
		try{
			var elm2 = document.getElementById(this._getElmId(this.__activeNode)).getElementsByTagName('a')[0];
			removecss(elm2,'active');
			elm2 = null;
		}
		catch(e){}
	}

	// Open parent node
	var iSPos;
	if ((iSPos = id.lastIndexOf('/')) > -1)
		this._open(id.substr(0,iSPos),'minus');
	else
		this._open(id,'minus');

	if (this._onactivate)
		this._onactivate(id);

	this.__activeNode = id;
	return true;
};


/**
 * @brief: return DOM Element ID
 **/
_me._getElmId = function (id){
	return this._pathName +'/'+ id;
};


/**
 * @brief: recursiv method for creating ul-li tree structure
 * @scope: PRIVATE
 * @date : 12.5.2006 11:50:37
 **/
_me.__row = function(aData,child){

	var path='',elm1,elm2,elm3,elm4,elm5,elm6,liclass,me = this;

	/* create main UL element */
	elm1 = mkElement("ul");
	if (!child){
		this.__idtable = {};
        var iTop = this._main.scrollTop;
		liclass = 'root';
	}
	else{
		path = child;
		liclass = 'none';
	}

	for(var i in aData){
		elm2 = mkElement("li",{"className":liclass + (aData[i]["liclass"]?' '+aData[i]["liclass"]:''),"id":this._pathName + '/' + path + i});
		elm3 = mkElement("div");

		if (aData[i]['arg'])
			this.__idtable[path + i] = aData[i]['arg'];

		/* [+] icon */
		elm5 = mkElement("span",{"className":'plussign'});
		elm5.innerHTML = '&nbsp;&nbsp;&nbsp;';
		elm3.appendChild(elm5);

		/* folder icon */
		elm6 = mkElement("span",{"className": (aData[i]['ico'] ? 'ico ' + aData[i]['ico'] : 'ico') });
		elm6.innerHTML = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
		elm3.appendChild(elm6);

		/* link */
		if (aData[i]["arg"]["disabled"])
			elm4 = mkElement("span",{"className":'inactive'});
		else
			elm4 = mkElement("a",{"href":'',"className":'treeitem'});
		
		if (aData[i]["itmclass"])
			addcss(elm4,aData[i]["itmclass"]);

		elm4.innerHTML = (aData[i]['title']?getLang(aData[i]['title'],null,true):(aData[i]['text']?aData[i]['text']:i.toString().entityify())) + (aData[i]['title2']?'<span>'+aData[i]['title2']+'</span>':'');  // Quote STRING !!!

		elm3.appendChild(elm4);

		elm2.appendChild(elm3);

		if (aData[i]['nodes'] && !Is.Empty(aData[i]['nodes'])){
			/* set "minus" className for node with childs */
			addcss(elm2,'plus');
			removecss(elm2,'none');

			/* parse and append child nodes */
			elm2.appendChild(this.__row(aData[i]['nodes'],path + i + '/'));
		}
		// n++;
		elm1.appendChild(elm2);
	}

	/* set "end" className for last node with childs */
	elm2.className += ' end';

	/* destruct html elements */
	elm2 = null; elm3 = null; elm4 = null; elm5 = null; elm6 = null;

	/* append root UL into _main */
	if (!child){

		this._main.innerHTML = '';
		this._main.appendChild(elm1);

		elm1.className = 'fTree';

		/* open previously opened nodes */
		if (count(this.__value))
			this._value(this.__value);
		else
		if (this._listener)
			this.__update(this._listener);

		// active
		if (this.__activeNode)
			this._setActive(this.__activeNode);

		this._main.scrollTop = iTop;
		return;
	}

	/* return UL element */
	return elm1;
};


/**
 * @brief: apply given CSS to node and all parent nodes,
 *         if node is not set than function works over all nodes in __value
 * @scope: PUBLIC
 * @date : 14.5.2006 15:58:10
 **/
_me._openup = function(css,id){
	var a = [],elm,pos,styl;

	if (!id)
		for(var i in this.__value)
		    a.push(this.__value[i]);
	else
		a.push(id);

	css = css == 'minus'?css:'plus';

	for (var i = 0; i<a.length;i++){
		/* remove wrong paths from array */
		if (!(elm = document.getElementById(this._getElmId(a[i])))){

			if ((pos = a[i].lastIndexOf('/'))>-1 && inArray(a,a[i].substr(0,pos))<0)
				a.push(a[i].substr(0,pos));

			a.splice(i,1);
			i--;
			continue;
		}

		if ((pos = a[i].lastIndexOf('/'))>-1 && inArray(a,a[i].substr(0,pos))<0)
			a.push(a[i].substr(0,pos));

		/* open node and parent nodes */
		if (elm.tagName != 'LI') continue;

		if (elm.className.indexOf(css) == -1){

			if (elm.className.indexOf('minus')>-1){
				removecss(elm,'minus');
				addcss(elm,css);
			}
			else
			if (elm.className.indexOf('plus')>-1){
				removecss(elm,'plus');
				addcss(elm,css);
			}
		}
	}
};

/**
 * @brief: open given node
 * @scope: PUBLIC
 * @param: ID    - Node id
 *         style - private recursive
 * @date : 14.5.2006 17:33:06
 **/
_me._open = function(id,style){
	if(!id) return;
	var css,elm,elmid = this._getElmId(id),a = id.split("/");

	/* check whether element exists */
	if (!(elm = document.getElementById(elmid))) return;

	/* close Node */
	if (elm.className.indexOf('plus')>-1 || style == 'minus'){

		/* add node into __value array */
		this.__value[getFreeKey(this.__value)] = id;
		this.__clear();
		/* open parent Nodes */
		this._openup('minus',id);
	}
	/* open node */
	else
	if (elm.className.indexOf('minus')>-1 || style == 'plus'){
		removecss(elm,'minus');
		addcss(elm,'plus');

		/* close node and all child nodes */
		var e;
		for(var i in this.__value){
//if (this.__value[i]==id || this.__value[i].indexOf(id+"/")==0)
			if (this.__value[i]==id){
				try{
					e = document.getElementById(this._pathName+"/"+this.__value[i]);
					if (e.className.indexOf('minus')>-1){
						removecss(e,'minus');
						addcss(e,'plus');
					}
				}
				catch(e){}
				delete this.__value[i];
				break; //pridano
			}
		}

		/* if node is not root node than add parent node path into array */
/*
		if (a.length>1) {
			var t = a.slice(0,-1).join("/");
			if (inArray(this.__value,t)==-1)
				this.__value[getFreeKey(this.__value)] = t;
		}
*/
	}
	else{
		elm = null;
		if (a.length>1)
			this._open(a.slice(0,-1).join("/"),'minus');
		return;
	}

	/* destruct elm var */
	elm = null;
	/* save opened threads into dataset */
	if (this._saveState && this._listener)
		this._saveme();

	if (this._treeCookie)
	{
		this.__clear();
		dataSet.add(this._treeCookie,'',clone(this._value(),true));
	}
};

/**
 * @brief: listener for tree structure
 **/
_me._listen_data = function(sDataSet,aDataPath){
	this._listener_data = sDataSet;
	if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
	dataSet.obey(this,'_listener_data',sDataSet);
};

_me._listen_cookie = function(sCookieName)
{
	this._treeCookie = sCookieName;

	var aCookieValue = dataSet.get(sCookieName);
	if (typeof aCookieValue == 'object')
		this._value(aCookieValue);
};

/**
 * @brief: update method
 **/
_me.__update = function(sDataSet,aDataPath){

	/* no update section */
	if (this._norefresh){
		this._updateBuffer = true;
		return false;
	}
	else
		this._updateBuffer = false;

	/* update tree structure */
	if (this._listener_data == sDataSet){

		if (aDataPath && aDataPath[2] == 'RECENT'){

			var elm;
			if((elm = document.getElementById(this._pathName+'/'+aDataPath[0]+'/'+aDataPath[1])) && (elm = elm.getElementsByTagName('A')[0]) && (elm = elm.getElementsByTagName('SPAN')[0])){
				var iRec = dataSet.get(sDataSet,aDataPath);
				if (parseInt(iRec)>0){
					addcss(elm.parentNode,'recent');
					elm.innerHTML = '&nbsp;('+iRec+')';
				}
				else{
					dataSet.remove(sDataSet,aDataPath,true);
                    removecss(elm.parentNode,'recent');
					elm.innerHTML = '';
				}
				
				elm = null;
				return;
			}
		}
		else
			this._fill();
	}

	/* update opened nodes */
	if (this._listener == sDataSet)
		this._value(dataSet.get(this._listener,this._listenerPath));
};


/**
 * @brief: return all open nodes (only last child in thread) as array of string paths
 * @scope: PUBLIC
 * @date : 14.5.2006 17:33:56
 **/
_me._value = function (v){
	if (v && typeof(v) == 'object') {
		/* close all nodes */
		if (this.__value.length) this._openup('plus');
		this.__value = this.__clear(v);
		/* open all nodes */
		this._openup('minus');
	}
	else
		this.__value = this.__clear(this.__value);

	return this.__value;
};

/**
 * @brief: removes spare paths from array
 * @scope: PRIVATE
 * @date : 14.5.2006 15:32:02
 **/
_me.__clear = function(a){
	if (!a)
		a = this.__value;

	a.sort().reverse();
	
	var pre;

	for(var i=0;i <= a.length;i++)
		if (typeof a[i] != 'undefined')
			if (typeof pre == 'undefined')
				pre=a[i];
			else
			if (pre==a[i])
				delete a[i];
			else
				pre = a[i];
	return a;
};

/**
 * @brief: optional onclick method
 * @param: e    - Event Handler (onclick event object)
 *         elm  - srcElement (link)
 *         id   - objects internal id (without pathName)
 *         arg  - item's arg section
 */
//_me._onclick = function(e,elm,id,arg){};
